// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// DeviceList class declaration
// **************************************

// **************************
// Headers
// **************************

#include "devicelist.h"
#include "mymessage.h"      // needed for error messages

// **************************
// Static variables
// **************************

Device DeviceList::dummy;

// **************************
// DeviceList functions
// **************************

DeviceList::DeviceList(const unsigned int l)
{  /// Constructor.

  size=l;
  if (size>0)
  {
    values=new Device[size];
  }
  else values=NULL;
  current=0;
}

DeviceList::DeviceList(const DeviceList& array)
{  /// Copy constructor.

  size=array.GetSize();
  if (size>0)
  {
    values=new Device[size];
    for (unsigned int i=0; i<size; i++)
    {
      values[i]=array(i);
    }
  }
  else values=NULL;
  current=array.GetCurrent();
}

DeviceList::~DeviceList()
{  /// Destructor.

  if (values) delete [] values;
  values=NULL;
}

DeviceList& DeviceList::operator=(const DeviceList& array)
{  /// Assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i]=array(i);
  }
  current=array.GetCurrent();
  return (*this);
}

void DeviceList::Set(const DeviceList& array)
{  /// Alternate assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i]=array(i);
  }
  current=array.GetCurrent();
}

void DeviceList::Clear(void)
{  /// Delete all values.

  if (values) delete [] values;
  values=NULL;
  size=0;
}

const Device& DeviceList::operator()(const unsigned int index) const
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::operator() const");
    return(dummy);
  }
  return (values[index]);
}

Device& DeviceList::operator()(const unsigned int index)
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::operator()");
    return(dummy);
  }
  return (values[index]);
}

void DeviceList::Append(const Device& value)
{  /// Append element.

  SetSize(size+1);
  values[size-1]=value;
}

void DeviceList::Append(const DeviceList& array)
{  /// Append all elements from other array.

  if (array.GetSize()==0) return;
  SetSize(size+array.GetSize());
  for (unsigned int i=0; i<array.GetSize(); i++)
  {
    values[(size-array.GetSize())+i]=array(i);
  }
}

const bool DeviceList::Insert(const unsigned int index, const Device& value)
{  /// Insert value at index.

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(value);
    return(false);
  }

  unsigned int i;
  Device *newValues=new Device[size+1];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
    newValues[index]=value;
  for (i=index; i<size; i++)
  {
    newValues[i+1]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size++;

  if (index<=current) current++;

  return(false);
}

const bool DeviceList::Insert(const unsigned int index, const DeviceList& array)
{  /// Insert array at index.

  if (array.GetSize()==0) return(false);

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(array);
    return(false);
  }

  unsigned int i;
  Device *newValues=new Device[size+array.GetSize()];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
  for (i=0; i<array.GetSize(); i++)
  {
    newValues[i]=array(i);
  }
  for (i=index; i<size; i++)
  {
    newValues[i+array.GetSize()]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size+=array.GetSize();

  if (index<=current) current+=array.GetSize();

  return(false);
}

const bool DeviceList::Delete(const unsigned int index, unsigned int width)
{ /// Delete width elements from index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Delete");
    return(true);
  }

  if (width==0) return(false);

  if (index+width>size) width=size-index;

  if (size==width)
  {
    delete [] values;
    values=NULL;
  }
  else
  {
    unsigned int i;
    Device *newValues=new Device[size-width];

    for (i=0; i<index; i++)
    {
      newValues[i]=values[i];
    }
    for (i=index+width; i<size; i++)
    {
      newValues[i-width]=values[i];
    }

    if (values) delete [] values;
    values=newValues;
    newValues=NULL;
  }

  size-=width;
  if (size==0) current=0;
  else if (current>=size) current=size-1;

  return(false);
}

const bool DeviceList::Set(const unsigned int index, const DeviceList& array)
{  /// Copy array elements to index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Set");
    return(true);
  }

  unsigned int i;

  if (index+array.GetSize()>size)
  {
    SetSize(index+array.GetSize());
    for (i=index; i<size; i++)
    {
      values[i]=array(i-index);
    }
  }
  else
  {
    for (i=index; i<array.GetSize()+index; i++)
    {
      values[i]=array(i-index);
    }
  }
  return(false);
}

void DeviceList::Extend(const unsigned int newsize)
{  /// Extend array to new size.

  if (newsize>size)
  {
    if (newsize>0)
    {
      Device *newValues=new Device[newsize];

      unsigned int i;

      for (i=0; i<size; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
  }
}

void DeviceList::Trunc(const unsigned int newsize)
{  /// Truncate array to new size.

  if (newsize<size)
  {
    if (newsize>0)
    {
      Device *newValues=new Device[newsize];

      for (unsigned int i=0; i<newsize; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
    if (size==0) current=0;
    else if (current>=size) current=size-1;
  }
}

void DeviceList::SetSize(const unsigned int newsize)
{  /// Set new array size.

  if (newsize<size) Trunc(newsize);
  else if (newsize>size) Extend(newsize);
  // if newsize==size there is nothing to do
}

const unsigned int DeviceList::GetSize(void) const
{  /// Return size of array.

  return (size);
}

const int DeviceList::Find(const MyString& name, const unsigned int start) const
{  /// Search device in list.

  int pos=-1;                     // -1 means not found

  for (unsigned int i=start; i<size; i++)
  {
    if (name==values[i].GetId())
    {
      pos=i;
      break;       // leave for-loop
    }
  }

  return(pos);
}

const unsigned int DeviceList::GetCurrent(void) const
{  /// Return current element.
  return(current);
}

const bool DeviceList::SetCurrent(const unsigned int index)
{  /// Set current element.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::SetCurrent");
    return(true);
  }
  current=index;
  return(false);
}

const Device& DeviceList::Current(void) const
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Current const");
    return(dummy);
  }
  return (values[current]);
}

Device& DeviceList::Current(void)
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DeviceList::Current");
    return(dummy);
  }
  return (values[current]);
}

const bool DeviceList::SetLast(void)
{  /// Set current element at last position.

  if (size==0) current=0;
  else current=size-1;
  return(false);
}

const bool DeviceList::DeleteCurrent(void)
{  /// Delete current element.

  if (size==0 || current>=size) return(false);
  return(Delete(current));
}

const bool DeviceList::Check(void)
{  /// Check if everything has correct syntax.

  unsigned int i, j;

  // check single
  for (i=0; i<size; i++)
  {
    if (values[i].Check())
    {
      MyMessage::Append("DeviceList::Check");
      current=i;
      return(true);
    }
  }

  // check if unique
  for (i=0; i<size; i++)
  {
    for (j=i+1; j<size; j++)
    {
      if (values[i].GetId()==values[j].GetId())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append("DeviceList::Check");
          return(true);
        }
        current=i;
        MyMessage::Set(ERROR_CHECK_SAME_DEVICE_ID, "DeviceList::Check");
        return(true);
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append("DeviceList::Check");
        return(true);
      }
    }
  }

  return(false);
}
