// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// DriverOptionList class declaration
// **************************************

// **************************
// Headers
// **************************

#include "driveroptionlist.h"
#include "mymessage.h"            // needed for error messages
#include <fstream>
#include <iostream>

using namespace std;

// **************************
// Static variables
// **************************

DriverOption DriverOptionList::dummy;

// **************************
// DriverOptionList functions
// **************************

DriverOptionList::DriverOptionList(const unsigned int l)
{  /// Constructor.

  size=l;
  if (size>0)
  {
    values=new DriverOption[size];
  }
  else values=NULL;
  current=0;

  lineNr=0;
}

DriverOptionList::DriverOptionList(const DriverOptionList& array)
{  /// Copy constructor.

  size=array.GetSize();
  if (size>0)
  {
    values=new DriverOption[size];
    for (unsigned int i=0; i<size; i++)
    {
      values[i]=array(i);
    }
  }
  else values=NULL;
  current=array.GetCurrent();

  lineNr=0;
}

DriverOptionList::~DriverOptionList()
{  /// Destructor.

  if (values) delete [] values;
  values=NULL;
}

DriverOptionList& DriverOptionList::operator=(const DriverOptionList& array)
{  /// Assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i]=array(i);
  }
  current=array.GetCurrent();

  lineNr=0;

  return (*this);
}

void DriverOptionList::Set(const DriverOptionList& array)
{  /// Alternate assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i]=array(i);
  }
  current=array.GetCurrent();
  lineNr=0;
}

void DriverOptionList::Clear(void)
{  /// Delete all values.

  if (values) delete [] values;
  values=NULL;
  size=0;
  current=0;
}

const DriverOption& DriverOptionList::operator()(const unsigned int index) const
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::operator() const");
    return(dummy);
  }
  return (values[index]);
}

DriverOption& DriverOptionList::operator()(const unsigned int index)
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::operator()");
    return(dummy);
  }
  return (values[index]);
}

const bool DriverOptionList::Append(const DriverOption& value)
{  /// Append element.

  if (Find(value.GetName(), true)!=-1)
  {
    MyMessage::Set(ERROR_DRIVERLIST_DOUBLE_OPTION, "DriverOptionList::Append");
    return(true);
  }

  SetSize(size+1);
  values[size-1]=value;
  return(false);
}

const bool DriverOptionList::Append(const DriverOptionList& array)
{  /// Append all elements from other array.

  if (array.GetSize()==0) return(false);

  unsigned int i;  // loop counter

  for (i=0; i<array.GetSize(); i++)
  {
    if (Find(array(i).GetName(), true)!=-1)
    {
      MyMessage::Set(ERROR_DRIVERLIST_DOUBLE_OPTION, "DriverOptionList::Append");
      return(true);
    }
    if (!MyMessage::OK())
    {
      MyMessage::Append("DriverOptionList::Append");
      return(true);
    }
  }

  SetSize(size+array.GetSize());
  for (unsigned int i=0; i<array.GetSize(); i++)
  {
    values[(size-array.GetSize())+i]=array(i);
  }
  return(false);
}

const bool DriverOptionList::Insert(const unsigned int index, const DriverOption& value)
{  /// Insert value at index.

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Insert");
    return(true);
  }

  if (index==size)
  {
    if (Append(value))
    {
      MyMessage::Append("DriverOptionList::Insert");
      return(true);
    }
    return(false);
  }

  unsigned int i;
  DriverOption *newValues=new DriverOption[size+1];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
    newValues[index]=value;
  for (i=index; i<size; i++)
  {
    newValues[i+1]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size++;

  if (index<=current) current++;

  return(false);
}

const bool DriverOptionList::Insert(const unsigned int index, const DriverOptionList& array)
{  /// Insert array at index.

  if (array.GetSize()==0) return(false);

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Insert");
    return(true);
  }

  if (index==size)
  {
    if (Append(array))
    {
      MyMessage::Append("DriverOptionList::Insert");
      return(true);
    }
    return(false);
  }

  unsigned int i;
  DriverOption *newValues=new DriverOption[size+array.GetSize()];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
  for (i=0; i<array.GetSize(); i++)
  {
    newValues[i]=array(i);
  }
  for (i=index; i<size; i++)
  {
    newValues[i+array.GetSize()]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size+=array.GetSize();

  if (index<=current) current+=array.GetSize();

  return(false);
}

const bool DriverOptionList::Delete(const unsigned int index, unsigned int width)
{ /// Delete width elements from index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Delete");
    return(true);
  }

  if (width==0) return(false);

  if (index+width>size) width=size-index;

  if (size==width)
  {
    delete [] values;
    values=NULL;
  }
  else
  {
    unsigned int i;
    DriverOption *newValues=new DriverOption[size-width];

    for (i=0; i<index; i++)
    {
      newValues[i]=values[i];
    }
    for (i=index+width; i<size; i++)
    {
      newValues[i-width]=values[i];
    }

    if (values) delete [] values;
    values=newValues;
    newValues=NULL;
  }

  size-=width;
  if (size==0) current=0;
  else if (current>=size) current=size-1;

  return(false);
}

const bool DriverOptionList::Set(const unsigned int index, const DriverOptionList& array)
{  /// Copy array elements to index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Set");
    return(true);
  }

  unsigned int i;

  if (index+array.GetSize()>size)
  {
    SetSize(index+array.GetSize());
    for (i=index; i<size; i++)
    {
      values[i]=array(i-index);
    }
  }
  else
  {
    for (i=index; i<array.GetSize()+index; i++)
    {
      values[i]=array(i-index);
    }
  }
  return(false);
}

void DriverOptionList::Extend(const unsigned int newsize)
{  /// Extend array to new size.

  if (newsize>size)
  {
    if (newsize>0)
    {
      DriverOption *newValues=new DriverOption[newsize];

      unsigned int i;

      for (i=0; i<size; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
  }
}

void DriverOptionList::Trunc(const unsigned int newsize)
{  /// Truncate array to new size.

  if (newsize<size)
  {
    if (newsize>0)
    {
      DriverOption *newValues=new DriverOption[newsize];

      for (unsigned int i=0; i<newsize; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
    if (size==0) current=0;
    else if (current>=size) current=size-1;
  }
}

void DriverOptionList::SetSize(const unsigned int newsize)
{  /// Set new array size.

  if (newsize<size) Trunc(newsize);
  else if (newsize>size) Extend(newsize);
  // if newsize==size there is nothing to do
}

const unsigned int DriverOptionList::GetSize(void) const
{  /// Return size of array.

  return (size);
}

const int DriverOptionList::Find(const MyString& name, const bool ignoreCase) const
{ // search option

  int pos=-1;

  for (unsigned int i=0; i<size; i++)
  {
    if (ignoreCase)
    {
      if (values[i].GetName().toLower()==name.toLower())
      {
        pos=i;
        break;
      }
    }
    else
    {
      if (values[i].GetName()==name)
      {
        pos=i;
        break;
      }
    }
  }

  return(pos);
}

const bool DriverOptionList::GetSection(DriverOptionList& array, const MyString& name) const
{  /// Select all elements from a special section.

  array.SetSize(0);  // clear array

  unsigned int number=0;
  unsigned int i;

  for (i=0; i<size; i++)
  {
    if (values[i].GetSection().toLower()==name.toLower()) number++;
  }

  array.SetSize(number);
  number=0;

  for (i=0; i<size; i++)
  {
    if (values[i].GetSection().toLower()==name.toLower())
    {
      array(number)=values[i];
      if (!MyMessage::OK())
      {
        MyMessage::Append("DriverOptionList::GetSection");
        return(true);
      }
      number++;
    }
  }
  return(false);

}

const unsigned int DriverOptionList::GetCurrent(void) const
{  /// Return current element.
  return(current);
}

const bool DriverOptionList::SetCurrent(const unsigned int index)
{  /// Set current element.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::SetCurrent");
    return(true);
  }
  current=index;
  return(false);
}

const DriverOption& DriverOptionList::Current(void) const
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Current const");
    return(dummy);
  }
  return (values[current]);
}

DriverOption& DriverOptionList::Current(void)
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "DriverOptionList::Current");
    return(dummy);
  }
  return (values[current]);
}

const bool DriverOptionList::SetLast(void)
{  /// Set current element at last position.

  if (size==0) current=0;
  else current=size-1;
  return(false);
}

const bool DriverOptionList::DeleteCurrent(void)
{  /// Delete current element.

  if (size==0 || current>=size) return(false);
  return(Delete(current));
}

const bool DriverOptionList::Check(void) const
{  /// Check if everything has correct syntax.

  unsigned int i, j;

  // check single
  for (i=0; i<size; i++)
  {
    if (values[i].Check())
    {
      MyMessage::Append("DriverOptionList::Check");
      return(true);
    }
  }

  // check if unique
  for (i=0; i<size; i++)
  {
    for (j=i+1; j<size; j++)
    {
      if (values[i].GetName().toLower()==values[j].GetName().toLower())
      {
        if (!MyMessage::OK())
        {
          MyMessage::Append("DriverOptionList::Check");
          return(true);
        }
        MyMessage::Set(ERROR_CHECK_SAME_DRIVEROPTION, "DriverOptionList::Check");
        return(true);
      }
      if (!MyMessage::OK())
      {
        MyMessage::Append("DriverOptionList::Check");
        return(true);
      }
    }
  }

  return(false);
}

const bool DriverOptionList::Load(const MyString& file, const bool reload)
{  // load driver option lists

  ifstream stream;                   // File access.
  char readline[1000];               // allocate memory for 1000 chars
  MyString line;                     // current line
  MyString orgLine;                  // original line
  MyString section;                  // current section
  bool comment=false;                // flag if a comment has started
  bool option=false;                 // flag if option has been started
  bool ignoreError=true;             // flag if errors should be ignored
  bool desc=false;                    // flag if description has started
  DriverOption tempOption;           // temporary option
  unsigned int tempLineNr=lineNr;
  lineNr=0;                          // start at beginning of file

  Clear();   // clear old list before

  stream.open(file.data());  // open file, but do not create it

  if (stream==NULL)
  {
    // Note: do not present any error
    // if the file is not defined we should not read it
    return(false);
  }

  while ( stream.getline(readline, 1000) )
  {
    lineNr++;
    if (!reload || lineNr>tempLineNr) ignoreError=false;

    orgLine=readline;     // set line

    line=orgLine;           // copy line (we need to change it)
    line.StripSpaces();   // strip spaces

    if (!line.IsEmpty())  // line is not empty
    {
      if (line.StartsWith("-->")) // end comment
      {
        comment=false;
      }
      else if (!comment)
      {
        if (line.StartsWith("<!--"))  // search for comment
        {
          comment=true;
        }
        else if (line.StartsWith("<?xml "))  // search for version
        {
          // ignore line
        }
        else if (line.StartsWith("</section>"))
        {  // section ends
          if (section.IsEmpty() && !ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_NO_SECTION, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }

          if (option)  // option not closed
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_TWO_OPTIONS, "DriverOptionList::Load", file);
              return(true);
            }
            else
            {
              if (!section.IsEmpty()) tempOption.SetSection(section);
              if (!tempOption.GetName().IsEmpty())
              {
                if (tempOption.GetValue()=="boolean" && tempOption.GetDefault().IsEmpty())
                {
                  tempOption.SetDefault("false");
                }

                if (Append(tempOption) && !ignoreError)     // add option to list
                {
                  MyMessage::Set(ERROR_DRIVERLIST_DOUBLE_OPTION, "DriverOptionList::Load", file, tempOption.GetName(), lineNr);
                  return(true);
                }
                tempOption.Clear();  // clear option again
                option=false;
                desc=false;
              }
            }
          }
          section.Empty();  // clear section
        } // option end
        else if (line.StartsWith("<section"))
        {  // section begins
          if (!section.IsEmpty())
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_TWO_SECTIONS, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            else  // the error should be ignored, we close the old option first
            {
              section.Empty();
            }
          }

          if (line.Last()!='>' && (!ignoreError))
          {  // line does not end with >
            MyMessage::Set(ERROR_DRIVERLIST_NO_TAG_END, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }
          // else ignore it

          line.StripRight(8);   // delete "<section" at start
          int pos=-1;

          // search name
          pos=line.Find("name=\"");
          if (pos>=0)
          {
            line.StripRight(pos+6);
            pos=line.Find('"');

            if (pos>=1)
            {
              section=line.Left(pos);

              if (section!="general" && section!="advanced" && section!="dual")
              {
                section.Empty();
                if (!ignoreError)
                {
                  MyMessage::Set(ERROR_DRIVERLIST_SECTION_NAME_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
                  return(true);
                }
              }
            }
            else if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_SECTION_NAME_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          }
          else if (!ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_SECTION_NAME_MISSING, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          } // name
        }
        else if (line.StartsWith("</option>"))
        {  // option ends

          if (!option && !ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_NO_OPTION, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }
          else if (desc && !ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_CLOSE_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }
          else
          {
            if (!section.IsEmpty()) tempOption.SetSection(section);
            if (!tempOption.GetName().IsEmpty())
            {
              if (tempOption.GetValue()=="boolean" && tempOption.GetDefault().IsEmpty())
              {
                tempOption.SetDefault("false");
              }

              if (Append(tempOption) && !ignoreError)     // add option to list
              {
                MyMessage::Set(ERROR_DRIVERLIST_DOUBLE_OPTION, "DriverOptionList::Load", file, tempOption.GetName(), lineNr);
                return(true);
              }
            }
          }

          tempOption.Clear();  // clear option again
          option=false;
          desc=false;
        } // option end
        else if (line.StartsWith("<option"))
        {  // option begins
          if (section.IsEmpty())
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_OPTION_WITHOUT_SECTION, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          }

          if (option)  // second option
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_TWO_OPTIONS, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            else  // the error should be ignored, we close the old option first
            {
              if (!section.IsEmpty()) tempOption.SetSection(section);
              if (!tempOption.GetName().IsEmpty())
              {
                if (tempOption.GetValue()=="boolean" && tempOption.GetDefault().IsEmpty())
                {
                  tempOption.SetDefault("false");
                }

                if (Append(tempOption) && !ignoreError)     // add option to list
                {
                  MyMessage::Set(ERROR_DRIVERLIST_DOUBLE_OPTION, "DriverOptionList::Load", file, tempOption.GetName(), lineNr);
                  return(true);
                }
              }
              tempOption.Clear();  // clear option again
              option=false;
              desc=false;
            }
          }

          if (line.Last()!='>' && !ignoreError)
          {  // line does not end with >
            MyMessage::Set(ERROR_DRIVERLIST_NO_TAG_END, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }
          // else ignore it

          line.StripRight(7);   // delete "<option" at start
          option=true;          // option has started

          MyString xLine;
          int pos=-1;

          // search name
          xLine=line;
          pos=xLine.Find("name=\"");
          if (pos>=0)
          {
            xLine.StripRight(pos+6);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetName(xLine.Left(pos));
            }
            else if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_OPTION_NAME_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            // else ignore error and do not set name
          }
          else if (!ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_OPTION_NAME_MISSING, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          } // name
          // else ignore error and do not set name

          // search value
          xLine=line;
          pos=xLine.Find("value=\"");
          if (pos>=0)
          {
            xLine.StripRight(pos+7);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              xLine=xLine.Left(pos).toLower();
              if (xLine=="string" || xLine=="integer" || xLine=="boolean")
              {
                tempOption.SetValue(xLine);
              }
              else if (!ignoreError)
              {
                MyMessage::Set(ERROR_DRIVERLIST_VALUE_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
                return(true);
              }
              // else error is ignored and value==string as standard
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_VALUE_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            // else ignore error
          } // value

          // search comment
          xLine=line;
          pos=xLine.Find("comment=\"");
          if (pos>=0)
          {
            xLine.StripRight(pos+9);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetComment(xLine.Left(pos));
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_COMMENT_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          } // comment

/*
          // search description
          xLine=line;
          pos=xLine.Find(_mT("description=\""));
          if (pos>=0)
          {
            xLine.StripRight(pos+13);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetDescription(xLine.Left(pos));
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_DESC_ERROR, _mT("DriverOptionList::Load"));
              return(true);
            }
          } // description

          xLine=line;
          pos=xLine.Find(_mT("desc=\""));
          if (pos>=0)
          {
            xLine.StripRight(pos+6);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetDescription(xLine.Left(pos));
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_DESC_ERROR, _mT("DriverOptionList::Load"));
              return(true);
            }
          } // description
*/

          // search strict
          xLine=line;
          pos=xLine.Find("strict=\"");
          if (pos>=0)
          {
            xLine.StripRight(pos+8);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              xLine=xLine.Left(pos);
              if (xLine=="true")
              {
                tempOption.SetStrict(true);
              }
              else if (xLine=="false")
              {
                tempOption.SetStrict(false);
              }
              else if (!ignoreError)
              {
                MyMessage::Set(ERROR_DRIVERLIST_STRICT_VALUE, "DriverOptionList::Load", file, orgLine, lineNr);
                return(true);
              }
              // else ignore error and strict is set to false
            }
            else if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_STRICT_ERROR, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            // else ignore error
          } // strict

/*
          // search section
          xLine=line;
          pos=xLine.Find(_mT("section=\""));
          if (pos>=0)
          {
            xLine.StripRight(pos+9);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              xLine=xLine.Left(pos);
              if (xLine==_mT("general"))
              {
                tempOption.SetSection(_mT("General"));
              }
              else if (xLine==_mT("advanced"))
              {
                tempOption.SetSection(_mT("Advanced"));
              }
              else if (xLine==_mT("dual"))
              {
                tempOption.SetSection(_mT("Dual-Monitoring / TV"));
              }
              else if (!ignoreError)
              {
                MyMessage::Set(ERROR_DRIVERLIST_SECTION_VALUE, _mT("DriverOptionList::Load"), file, orgLine, lineNr);
                return(true);
              }
              // else ignore error and section is General
            }
            else if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_SECTION_ERROR, _mT("DriverOptionList::Load"), file, orgLine, lineNr);
              return(true);
            }
            // else ignore error
          } // section
*/

        } // option begins
        else if (line.StartsWith("<default>"))
        {  // default argument

          if (!option)  // no option started
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_NO_OPTION,"DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          }
          else // option started
          {
            if (desc && (!ignoreError))
            {
              MyMessage::Set(ERROR_DRIVERLIST_CLOSE_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }

            if (!tempOption.GetDefault().IsEmpty() && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_TWO_DEFAULT, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }

            line.StripRight(9);
            int pos=line.Find("</default>");

            if (pos==-1 && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_NO_DEFAULT_END, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            line.StripLeft(10);

            if (line.IsEmpty() && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_EMPTY_DEFAULT, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }

            tempOption.SetDefault(line);
          } // option started
        } // default begin


        else if (line.StartsWith("<argument>"))
        {  // default argument

          if (!option)  // no option started
          {
            if ( !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_NO_OPTION, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          }
          else // option started
          {
            if (desc && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_CLOSE_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }

            line.StripRight(10);
            int pos=line.Find("</argument>");

            if (pos==-1 && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_NO_ARGUMENT_END, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }

            line.StripLeft(11);

            if (line.IsEmpty() && !ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_EMPTY_ARGUMENT, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
            tempOption.AddArgument(line);
          } // option started
        } // argument begin

        else if (line.StartsWith("<description>") || line.StartsWith("<desc>"))
        {  // default argument

          if (!option)  // no option started
          {
            if (!ignoreError)
            {
              MyMessage::Set(ERROR_DRIVERLIST_NO_OPTION, "DriverOptionList::Load", file, orgLine, lineNr);
              return(true);
            }
          }
          else // option started
          {
            if (line.StartsWith("<description>"))
            {
              line.StripRight(13);
              if (line.Find("</description>")>=0)
              {
                line.StripLeft(14);

                if (line.IsEmpty() && !ignoreError)
                {
                  MyMessage::Set(ERROR_DRIVERLIST_EMPTY_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
                  return(true);
                }
              }
              else desc=true;
            }
            else if (line.StartsWith("<desc>"))
            {
              line.StripRight(6);
              if (line.Find("</desc>")>=0)
              {
                line.StripLeft(7);

                if (line.IsEmpty() && !ignoreError)
                {
                  MyMessage::Set(ERROR_DRIVERLIST_EMPTY_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
                  return(true);
                }
                tempOption.SetDescription(line);
              }
              else desc=true;
            }
            tempOption.SetDescription(line);

          } // option started
        } // description begin

        else if (line.StartsWith("</description>") || line.StartsWith("</desc>"))
        {  // description ends

          if (!desc && !ignoreError)
          {
            MyMessage::Set(ERROR_DRIVERLIST_NO_DESC, "DriverOptionList::Load", file, orgLine, lineNr);
            return(true);
          }

          desc=false;
        }

/*
          // search description
          xLine=line;
          pos=xLine.Find(_mT("description=\""));
          if (pos>=0)
          {
            xLine.StripRight(pos+13);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetDescription(xLine.Left(pos));
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_DESC_ERROR, _mT("DriverOptionList::Load"));
              return(true);
            }
          } // description

          xLine=line;
          pos=xLine.Find(_mT("desc=\""));
          if (pos>=0)
          {
            xLine.StripRight(pos+6);
            pos=xLine.Find('"');
            if (pos>=1)
            {
              tempOption.SetDescription(xLine.Left(pos));
            }
            else if (!ignoreError)
            {  // error
              MyMessage::Set(ERROR_DRIVERLIST_DESC_ERROR, _mT("DriverOptionList::Load"));
              return(true);
            }
          } // description
*/
        else if (desc)
        {
          if (line.Find("</description>")>=0)
          {
            line.StripLeft(14);
            desc=false;
          }
          else if (line.Find("</desc>")>=0)
          {
            line.StripLeft(7);
            desc=false;
          }

          if (!line.IsEmpty())
          {
            if (tempOption.GetDescription().IsEmpty())
            {
              tempOption.SetDescription(line);
            }
            else
            {
              line=tempOption.GetDescription()+'\n'+line;
              tempOption.SetDescription(line);
            }
          }
        }

        else if (!ignoreError) // unrecognized entry
        {
          MyMessage::Set(ERROR_DRIVERLIST_UNKNOWN, "DriverOptionList::Load", file, orgLine, lineNr);
          return(true);
        }

      } // no comment
    } // empty line
  } // end of file

  stream.close();

  return(false);
}

