// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// DriverOptionList class definitions
// **************************************

#if !defined(DRIVEROPTIONLIST_H)
#define DRIVEROPTIONLIST_H

// **************************
// Headers
// **************************

#include "driveroption.h"

// **************************
// DriverOptionList class
// **************************

class DriverOptionList {

public:
  /// Constructor.
  /**
  * Create array with <i>l</i> elements.
  */
  DriverOptionList(const unsigned int l=0);

  /// Copy constructor.
  /**
  * Copy all array elements.
  */
  DriverOptionList(const DriverOptionList& array);

  /// Destructor.
  /**
  * Free memory.
  */
  virtual ~DriverOptionList();

  /// Assignment operator.
  /**
  * Copy all array elements.
  */
  DriverOptionList& operator=(const DriverOptionList& array);

  /// Delete all values.
  void Clear(void);

  /// Alternate assignment operator.
  /**
  * Copy all array elements.
  */
  void Set(const DriverOptionList& array);

  /// Access operator.
  /**
  * Return entry or dummy if <i>index</i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  const DriverOption& operator()(const unsigned int index) const;

  /// Access operator (as reference).
  /**
  * Return entry or dummy if <i><b>index</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  DriverOption& operator()(const unsigned int index);

  /// Append element.
  /**
  * Return <b>true</b> if the value already exist else <b>false</b>.
  */
  const bool Append(const DriverOption& value);

  /// Append all elements from other array.
  /**
  * Return <b>true</b> if one value already exist else <b>false</b>.
  */
  const bool Append(const DriverOptionList& array);

  /// Insert value at index.
  /**
  * If <i>index == size</i> the value will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const DriverOption&)
  */
  const bool Insert(const unsigned int index, const DriverOption& value);

  /// Insert array at index.
  /**
  * If <i>index == size</i> the array will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const DriverOptionList&)
  */
  const bool Insert(const unsigned int index, const DriverOptionList& array);

  /// Delete <i>width</i> elements from <i>index</i>.
  /**
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Delete(const unsigned int index, unsigned int width=1);

  /// Copy array elements to <i>index</i>.
  /**
  * If <i>size &lt; index+array.size</i>, the array will be extended.
  *
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Set(const unsigned int index, const DriverOptionList& array);

  /// Extend array to new size.
  void Extend(const unsigned int newsize);

  /// Truncate array to new size.
  void Trunc(const unsigned int newsize);

  /// Set new array size.
  /**
  * See also: Extend(const unsigned int), Trunc(const unsigned int)
  */
  void SetSize(const unsigned int newsize);

  /// Return size of array.
  const unsigned int GetSize(void) const;

  /// Load driver option list.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool Load(const MyString& file, const bool reload=true);

  /// Find an option in the array.
  /**
  * Return position of element or <b>-1</b> if not found.
  */
  const int Find(const MyString& name, const bool ignoreCase) const;

  /// Select all elements from a special section.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool GetSection(DriverOptionList& array, const MyString& name) const;

  /// Return current element.
  const unsigned int GetCurrent(void) const;

  /// Set current element.
  /**
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool SetCurrent(const unsigned int index);

  /// Return current element.
  /**
  * Return entry or dummy if <i>current</i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  const DriverOption& Current(void) const;

  /// Return current element (as reference).
  /**
  * Return entry or dummy if <i><b>current</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  DriverOption& Current(void);

  /// Set current element at last position.
  /**
  * Return <b>false</b> every time. It may be necessary
  * to return <b>true</b> if <i>size==0</i>.
  */
  const bool SetLast(void);

  /// Delete current element.
  /**
  * Return <b>false</b> every time. It may be necessary
  * to return <b>true</b> if <i>size==0</i>.
  */
  const bool DeleteCurrent(void);

  /// Check if everything has correct syntax.
  /**
  * Return <b>true</b> if something is wrong else <b>false</b>.
  */
  const bool Check(void) const;

private:
  DriverOption* values;        ///< Array elements.
  unsigned int size;           ///< Size of array.
  unsigned int current;     ///< Current element.

  unsigned int lineNr;         ///< Line counter for reading file.

  /// Dummy element.
  /**
  * This element is necessary for returning a reference
  * if the array access is out of boundaries.
  *
  * See also: operator()(const unsigned int)
  */
  static DriverOption dummy;

};

#endif // !defined(DRIVEROPTIONLIST_H)

