// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// ModuleList class declaration
// **************************************

// **************************
// Headers
// **************************

#include "modulelist.h"
#include "mymessage.h"      // needed for error messages

// **************************
// Static variables
// **************************

Module ModuleList::dummy;

// **************************
// ModuleList functions
// **************************

ModuleList::ModuleList(const unsigned int l)
{  /// Constructor.

  size=l;
  if (size>0)
  {
    values=new Module[size];
  }
  else values=NULL;
  current=0;
}

ModuleList::ModuleList(const ModuleList& array)
{  /// Copy constructor.

  size=array.GetSize();
  if (size>0)
  {
    values=new Module[size];
    for (unsigned int i=0; i<size; i++)
    {
      values[i]=array(i);
    }
  }
  else values=NULL;
  current=array.GetCurrent();

  comments=array.Comments();
}

ModuleList::~ModuleList()
{  /// Destructor.

  if (values) delete [] values;
  values=NULL;
}

ModuleList& ModuleList::operator=(const ModuleList& array)
{  /// Assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
      values[i]=array(i);
  }
  current=array.GetCurrent();
  comments=array.Comments();
  return (*this);
}

void ModuleList::Set(const ModuleList& array)
{  /// Alternate assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i]=array(i);
  }
  current=array.GetCurrent();
  comments=array.Comments();
}

void ModuleList::Clear(void)
{  /// Delete all values.

  if (values) delete [] values;
  values=NULL;
  size=0;
  current=0;
  comments.Clear();
}

const Module& ModuleList::operator()(const unsigned int index) const
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::operator() const");
    return(dummy);
  }
  return (values[index]);
}

Module& ModuleList::operator()(const unsigned int index)
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::operator()");
    return(dummy);
  }
  return (values[index]);
}

void ModuleList::Append(const Module& value)
{  /// Append element.

  SetSize(size+1);
  values[size-1]=value;
}

void ModuleList::Append(const ModuleList& array)
{  /// Append all elements from other array.

  if (array.GetSize()==0) return;
  SetSize(size+array.GetSize());
  for (unsigned int i=0; i<array.GetSize(); i++)
  {
    values[(size-array.GetSize())+i]=array(i);
  }
}

const bool ModuleList::Insert(const unsigned int index, const Module& value)
{  /// Insert value at index.

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(value);
    return(false);
  }

  unsigned int i;
  Module *newValues=new Module[size+1];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
    newValues[index]=value;
  for (i=index; i<size; i++)
  {
    newValues[i+1]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size++;

  if (index<=current) current++;

  return(false);
}

const bool ModuleList::Insert(const unsigned int index, const ModuleList& array)
{  /// Insert array at index.

  if (array.GetSize()==0) return(false);

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(array);
    return(false);
  }

  unsigned int i;
  Module *newValues=new Module[size+array.GetSize()];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
  for (i=0; i<array.GetSize(); i++)
  {
    newValues[i]=array(i);
  }
  for (i=index; i<size; i++)
  {
    newValues[i+array.GetSize()]=values[i];
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size+=array.GetSize();

  if (index<=current) current+=array.GetSize();

  return(false);
}

const bool ModuleList::Delete(const unsigned int index, unsigned int width)
{ /// Delete width elements from index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Delete");
    return(true);
  }

  if (width==0) return(false);

  if (index+width>size) width=size-index;

  if (size==width)
  {
    delete [] values;
    values=NULL;
  }
  else
  {
    unsigned int i;
    Module *newValues=new Module[size-width];

    for (i=0; i<index; i++)
    {
      newValues[i]=values[i];
    }
    for (i=index+width; i<size; i++)
    {
      newValues[i-width]=values[i];
    }

    if (values) delete [] values;
    values=newValues;
    newValues=NULL;
  }

  size-=width;
  if (size==0) current=0;
  else if (current>=size) current=size-1;

  return(false);
}

const bool ModuleList::Set(const unsigned int index, const ModuleList& array)
{  /// Copy array elements to index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Set");
    return(true);
  }

  unsigned int i;

  if (index+array.GetSize()>size)
  {
    SetSize(index+array.GetSize());
    for (i=index; i<size; i++)
    {
      values[i]=array(i-index);
    }
  }
  else
  {
    for (i=index; i<array.GetSize()+index; i++)
    {
      values[i]=array(i-index);
    }
  }
  return(false);
}

void ModuleList::Extend(const unsigned int newsize)
{  /// Extend array to new size.

  if (newsize>size)
  {
    if (newsize>0)
    {
      Module *newValues=new Module[newsize];

      unsigned int i;

      for (i=0; i<size; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
  }
}

void ModuleList::Trunc(const unsigned int newsize)
{  /// Truncate array to new size.

  if (newsize<size)
  {
    if (newsize>0)
    {
      Module *newValues=new Module[newsize];

      for (unsigned int i=0; i<newsize; i++)
      {
        newValues[i]=values[i];
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
    if (size==0) current=0;
    else if (current>=size) current=size-1;
  }
}

void ModuleList::SetSize(const unsigned int newsize)
{  /// Set new array size.

  if (newsize<size) Trunc(newsize);
  else if (newsize>size) Extend(newsize);
  // if newsize==size there is nothing to do
}

const unsigned int ModuleList::GetSize(void) const
{  /// Return size of array.

  return (size);
}

const StringArray& ModuleList::Comments(void) const
{  // return comments
  return(comments);
}

StringArray& ModuleList::Comments(void)
{  // return comments as reference
  return(comments);
}

const unsigned int ModuleList::GetCurrent(void) const
{  /// Return current element.
  return(current);
}

const bool ModuleList::SetCurrent(const unsigned int index)
{  /// Set current element.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::SetCurrent");
    return(true);
  }
  current=index;
  return(false);
}

const Module& ModuleList::Current(void) const
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Current const");
    return(dummy);
  }
  return (values[current]);
}

Module& ModuleList::Current(void)
{  /// Access operator.

  if (current>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "ModuleList::Current");
    return(dummy);
  }
  return (values[current]);
}

const bool ModuleList::SetLast(void)
{  /// Set current element at last position.

  if (size==0) current=0;
  else current=size-1;
  return(false);
}

const bool ModuleList::DeleteCurrent(void)
{  /// Delete current element.

  if (size==0 || current>=size) return(false);
  return(Delete(current));
}

const bool ModuleList::Check(void)
{  /// Check if everything has correct syntax.

  unsigned int i,j;

  // check single
  for (i=0; i<size; i++)
  {
    if (values[i].Check())
    {
      current=i;
      MyMessage::Append("ModuleList::Check");
      return(true);
    }
  }

  // check if unique
  for (i=0; i<size; i++)
  {
    if (values[i].IsActive())
    {
      for (j=i+1; j<size; j++)
      {
        if (values[j].IsActive() && values[i].GetName()==values[j].GetName())
        {
          if (!MyMessage::OK())
          {
            MyMessage::Append("ModuleList::Check");
            return(true);
          }
          current=i;
          MyMessage::Set(ERROR_CHECK_SAME_MODULE, "ModuleList::Check");
          return(true);
        }
        if (!MyMessage::OK())
        {
          MyMessage::Append("ModuleList::Check");
          return(true);
        }
      }
    }
  }

  return(false);
}

