// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// Array classes declaration
// **************************************

// **************************
// Headers
// **************************

#include "stringarray.h"
#include "mymessage.h"     // needed for error messages

// **************************
// Static variables
// **************************

MyString StringArray::dummy;

// **************************
// StringArray Functions
// **************************

StringArray::StringArray(const unsigned int l)
{  /// Constructor.

  size=l;
  if (size>0)
  {
    values=new MyString[size];
  }
  else values=NULL;
}

StringArray::StringArray(const StringArray& array)
{  /// Copy constructor.

  size=array.GetSize();
  if (size>0)
  {
    values=new MyString[size];
    for (unsigned int i=0; i<size; i++)
    {
      values[i].Set(array(i));
    }
  }
  else values=NULL;
}

StringArray::~StringArray()
{  /// Destructor.

  if (values) delete [] values;
  values=NULL;
}

StringArray& StringArray::operator=(const StringArray& array)
{  /// Assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i].Set(array(i));
  }
  return (*this);
}

void StringArray::Set(const StringArray& array)
{  /// Alternate assignment operator.

  SetSize(array.GetSize());
  for (unsigned int i=0; i<size; i++)
  {
    values[i].Set(array(i));
  }
}

const bool StringArray::operator==(const StringArray& array) const
{  /// Compare operator.

  bool equal=true;

  if (size!=array.GetSize())  // size not equal
  {
    equal=false;
  }
  else
  {
    for (unsigned int i=0;i<size;i++)
    {
      if (values[i]==array(i))
      {
        equal=false;
        break;
      }
    }
  }

  return (equal);
}

const bool StringArray::operator!=(const StringArray& array) const
{  /// Compare operator.

  bool unequal=false;

  if (size!=array.GetSize())  // size not equal
  {
    unequal=false;
  }
  else
  {
    for (unsigned int i=0;i<size;i++)
    {
      if (values[i]!=array(i))
      {
        unequal=true;
        break;
      }
    }
  }

  return (unequal);
}

const MyString& StringArray::operator()(const unsigned int index) const
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "StringArray::operator() const");
    return(dummy);
  }
  return (values[index]);
}

MyString& StringArray::operator()(const unsigned int index)
{  /// Access operator.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "StringArray::operator()");
    return(dummy);
  }
  return (values[index]);
}

void StringArray::Append(const MyString& value)
{  /// Append element.

  SetSize(size+1);
  values[size-1].Set(value);
}

void StringArray::Append(const StringArray& array)
{  /// Append all elements from other array.

  if (array.GetSize()==0) return;
  SetSize(size+array.GetSize());
  for (unsigned int i=0; i<array.GetSize(); i++)
  {
    values[(size-array.GetSize())+i].Set(array(i));
  }
}

const bool StringArray::Insert(const unsigned int index, const MyString& value)
{  /// Insert value at index.

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE,"StringArray::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(value);
    return(false);
  }

  unsigned int i;
  MyString *newValues=new MyString[size+1];

  for (i=0; i<index; i++)
  {
    newValues[i].Set(values[i]);
  }
  newValues[index].Set(value);
  for (i=index; i<size; i++)
  {
    newValues[i+1].Set(values[i]);
  }

  if (values) delete [] values;
  values=newValues;
  newValues=NULL;

  size++;

  return(false);
}

const bool StringArray::Insert(const unsigned int index, const StringArray& array)
{  /// Insert array at index.

  if (array.GetSize()==0) return(false);

  if (index>size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "StringArray::Insert");
    return(true);
  }

  if (index==size)
  {
    Append(array);
    return(false);
  }

  unsigned int i;
  MyString *newValues=new MyString[size+array.GetSize()];

  for (i=0; i<index; i++)
  {
    newValues[i]=values[i];
  }
  for (i=0; i<array.GetSize(); i++)
  {
    newValues[i]=array(i);
  }
  for (i=index; i<size; i++)
  {
   newValues[i+array.GetSize()]=values[i];
  }

  if (values) delete [] values;    // give memory free
  values=newValues;
  newValues=NULL;

  size+=array.GetSize();

  return(false);
}

const bool StringArray::Delete(const unsigned int index, unsigned int width)
{ /// Delete width elements from index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "StringArray::Delete");
    return(true);
  }

  if (width==0) return(false);

  if (index+width>size) width=size-index;

  if (size==width)
  {
    delete [] values;
    values=NULL;
  }
  else
  {
    unsigned int i;
    MyString *newValues=new MyString[size-width];

    for (i=0; i<index; i++)
    {
      newValues[i].Set(values[i]);
    }
    for (i=index+width; i<size; i++)
    {
      newValues[i-width].Set(values[i]);
    }

    if (values) delete [] values;
    values=newValues;
    newValues=NULL;
  }

  size-=width;
  return(false);
}

const bool StringArray::Set(const unsigned int index, const StringArray& array)
{  /// Copy array elements to index.

  if (index>=size)
  {
    MyMessage::Set(ERROR_INDEX_OUT_OF_RANGE, "StringArray::Set");
    return(true);
  }

  unsigned int i;

  if (index+array.GetSize()>size)
  {
    SetSize(index+array.GetSize());
    for (i=index; i<size; i++)
    {
      values[i].Set(array(i-index));
    }
  }
  else
  {
    for (i=index; i<array.GetSize()+index; i++)
    {
      values[i].Set(array(i-index));
    }
  }
  return(false);
}

void StringArray::Extend(const unsigned int newsize)
{  /// Extend array to new size.

  if (newsize>size)
  {
    if (newsize>0)
    {
      MyString *newValues=new MyString[newsize];

      for (unsigned int i=0; i<size; i++)
      {
        newValues[i].Set(values[i]);
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
  }
}

void StringArray::Trunc(const unsigned int newsize)
{  /// Truncate array to new size.

  if (newsize<size)
  {
    if (newsize>0)
    {
      MyString *newValues=new MyString[newsize];

      for (unsigned int i=0; i<newsize; i++)
      {
        newValues[i].Set(values[i]);
      }

      if (values) delete [] values;
      values=newValues;
      newValues=NULL;
    }
    else
    {
      if (values) delete [] values;
      values=NULL;
    }
    size=newsize;
  }
}

void StringArray::SetSize(const unsigned int newsize)
{  /// Set new array size.

  if (newsize<size) Trunc(newsize);
  else if (newsize>size) Extend(newsize);
  // if newsize==size there is nothing to do
}

const unsigned int StringArray::GetSize(void) const
{  /// Return size of array.

  return (size);
}

const int StringArray::Find(const MyString& value, const unsigned int start, const bool ignoreCase) const
{  /// Find an element in the array.

  int pos=-1;                     // -1 means not found

  for (unsigned int i=start; i<size; i++)
  {
    if (ignoreCase)
    {
      if (value.toLower()==values[i].toLower())
      {
        pos=i;
        break;       // leave for-loop
      }
    }
    else
    {
      if (value==values[i])
      {
        pos=i;
        break;       // leave for-loop
      }
    }
  }

  return(pos);
}

void StringArray::Clear(void)
{  /// Delete all values.

  if (values) delete [] values;
  values=0;
  size=0;

}


