// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// **************************************
// MyString Array classes definitions
// **************************************

#if !defined(STRINGARRAY_H)
#define STRINGARRAY_H

// **************************
// Headers
// **************************

#include "mystring.h"

// **************************
// StringArray Class
// **************************

class StringArray {

friend class TextFile;

public:
  /// Constructor.
  /**
  * Create array with <i>l</i> elements.
  */
  StringArray(const unsigned int l=0);

  /// Copy constructor
  /**
  * Copy all array elements.
  */
  StringArray(const StringArray& array);

  /// Destructor.
  /**
  * Free memory.
  */
  virtual ~StringArray();

  /// Assignment operator.
  /**
  * Copy all array elements.
  */
  StringArray& operator=(const StringArray& array);

  /// Alternate assignment operator.
  /**
  * Copy all array elements.
  */
  void Set(const StringArray& array);

  /// Compare operator.
  /**
  * Compare all elements.
  *
  * Return <b>true</b> if all elements are equal else <b>false</b>.
  */
  const bool operator==(const StringArray& array) const;

  /// Compare operator.
  /**
  * Compare all elements.
  *
  * Return <b>true</b> if one element is different else <b>false</b>.
  */
  const bool operator!=(const StringArray& array) const;

  /// Access operator.
  /**
  * Return entry or dummy if <i><b>index</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  const MyString& operator()(const unsigned int index) const;

  /// Access operator (as reference).
  /**
  * Return entry or dummy if <i><b>index</b></i> is out of boundaries.
  *
  * <b>Important:</b> You should check MyMessage::OK() after calling this function.
  */
  MyString& operator()(const unsigned int index);

  /// Append element.
  void Append(const MyString& value);

  /// Append all elements from other array.
  void Append(const StringArray& array);

  /// Insert value at index.
  /**
  * If <i>index == size</i> the value will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const MyString&)
  */
  const bool Insert(const unsigned int index, const MyString& value);

  /// Insert array at index.
  /**
  * If <i>index == size</i> the array will be appended.
  *
  * Return <b>true</b> if <i>index &gt; size</i> else <b>false</b>.
  *
  * See also: Append(const StringArray&)
  */
  const bool Insert(const unsigned int index, const StringArray& array);

  /// Delete <i>width</i> elements from <i>index</i>.
  /**
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Delete(const unsigned int index, unsigned int width=1);

  /// Copy array elements to <i>index</i>.
  /**
  * If <i>size &lt; index+array.size</i>, the array will be extended.
  *
  * Return <b>true</b> if <i>index &gt;= size</i> else <b>false</b>.
  */
  const bool Set(const unsigned int index, const StringArray& array);

  /// Extend array to new size.
  void Extend(const unsigned int newsize);

  /// Truncate array to new size.
  void Trunc(const unsigned int newsize);

  /// Set new array size.
  /**
  * See also: Extend(const unsigned int), Trunc(const unsigned int)
  */
  void SetSize(const unsigned int newsize);

  /// Return size of array.
  const unsigned int GetSize(void) const;

  /// Find an element in the array.
  /**
  * Return position of element or <b>-1</b> if not found.
  */
  const int Find(const MyString& value, const unsigned int start=0, const bool ignoreCase=false) const;

  /// Delete all values.
  void Clear(void);

private:
  MyString* values;        ///< Array elements.
  unsigned int size;        ///< Size of array.

  /// Dummy element.
  /**
  * This element is necessary for returning a reference
  * if the array access is out of boundaries.
  */
  static MyString dummy;
};

#endif // !defined(STRINGARRAY_H)
