// *************************************************************************
// libxorgedit
// Copyright 2006, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// Licence: GPL
// *************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef XORGFILE_H
#define XORGFILE_H

// **************************************
// Header Files
// **************************************

#include "optionlist.h"
#include "devicelist.h"
#include "extensionlist.h"
#include "filelist.h"
#include "inputdevicelist.h"
#include "modulelist.h"
#include "monitorlist.h"
#include "screenlist.h"
#include "serverflaglist.h"
#include "serverlayoutlist.h"
#include "drimode.h"
#include "textfile.h"

// **************************
// Constants
// **************************

enum xorgSectionID
{ // IDs for sections in xorg.conf

  XORG_FILES=100,
  XORG_SERVERFLAGS,
  XORG_MODULE,
  XORG_INPUTDEVICE,
  XORG_DEVICE,
  XORG_MONITOR,
  XORG_SCREEN,
  XORG_SERVERLAYOUT,
  XORG_DRI,
  XORG_EXTENSIONS,
  XORG_DISPLAY,
  XORG_NOSECTION,
  XORG_VIDEOADAPTOR,    // not recognized yet (and ever)
  XORG_VENDOR,          // not recognized yet (and ever?)
};


// **************************
// Class Definition
// **************************

class XorgFile
{
public:
  /// Constructor.
  XorgFile(void);

  /// Copy Constructor.
  XorgFile(const XorgFile& xorg);

  /// Destructor.
  virtual ~XorgFile();

  /// Assignment operator.
  XorgFile& operator=(const XorgFile& xorg);

  /// Get devices.
  const DeviceList& Devices(void) const;

  /// Get input devices.
  const InputDeviceList& InputDevices(void) const;

  /// Get monitors.
  const MonitorList& Monitors(void) const;

  /// Get screens.
  const ScreenList& Screens(void) const;

  /// Get server layouts.
  const ServerLayoutList& ServerLayouts(void) const;

  /// Get server flags.
  const ServerFlagList& ServerFlags(void) const;

  /// Get files.
  const FileList& Files(void) const;

  /// Get modules.
  const ModuleList& Modules(void) const;

  /// Get extension.
  const ExtensionList& Extensions(void) const;

  /// Get dri mode.
  const DriMode& Dri(void) const;

  /// Get comments.
  const StringArray& Comments(void) const;

  /// Get devices (as reference).
  DeviceList& Devices(void);

  /// Get input devices (as reference).
  InputDeviceList& InputDevices(void);

  /// Get monitors (as reference).
  MonitorList& Monitors(void);

  /// Get screen (as reference).
  ScreenList& Screens(void);

  /// Get server layouts (as reference).
  ServerLayoutList& ServerLayouts(void);

  /// Get server flags (as reference).
  ServerFlagList& ServerFlags(void);

  /// Get files (as reference).
  FileList& Files(void);

  /// Get modules (as reference).
  ModuleList& Modules(void);

  /// Get extension (as reference).
  ExtensionList& Extensions(void);

  /// Get dri mode (as reference).
  DriMode& Dri(void);

  /// Get comments (as reference).
  StringArray& Comments(void);

  /// Clear everything.
  void Clear(void);

  /// Load xorg.conf.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool Load(const MyString& name, const bool reload);

  /// Save xorg.conf.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool Save(const MyString& name, const MyString& version);

  /// Preview xorg.conf.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool Preview(TextFile& text, const MyString& name, const MyString& version);

  /// Check if everything is correct.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool CheckAll(void);

private:
  /// Extract argument from line.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool GetArgument(MyString &arg, const MyString &line) const;

  /// Extract option from line.
  /**
  * Return <b>true</b> if an error occured else <b>false</b>.
  */
  const bool GetOption(Option& opt, const MyString &line) const;

  /// Remove comments in line and store them separately.
  /**
  * Return <b>true</b> if whole line is a comment else <b>false</b>.
  */
  const bool FreeComments(MyString& line, MyString& comment) const;

  unsigned int lineNr; ///< line counter

  FileList files;
  ModuleList modules;
  ServerLayoutList serverLayouts;
  InputDeviceList inputDevices;
  DeviceList devices;
  MonitorList monitors;
  ScreenList screens;
  ServerFlagList serverFlags;
  ExtensionList extensions;
  DriMode driEntry;
  StringArray comments;

  MyString optDir;   ///< Option Directory.

};

#endif // XORGFILE_H
